# ----------------------------------------------------------------------------
#       Copyright (C) 2013-2014 Huynh Vi Lam  <domovilam@gmail.com>
#
#       This file is part of pimucha.
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

import logging
from piHAlibs import (debug,log2console,log2db,log2file,printer)
from .corelibs import (chkparsdaemon)
from .servers import (SetupDaemons)

logger = logging.getLogger()


DFEATURES = {
    'DEBUG':debug,
    'LOG2DB':log2db,
    'LOG2FILE':log2file,
    }


FTARGETS = {
    }


def chkdfeature(params):
    # params = [name:ctrl,decname,feature(dargs)]
    if len(params) != 3:
        logger.error('Three parameters needed for decoded feature')
        return None
    p0 = chkparsdaemon(params[0],SetupDaemons)
    if not p0:
        logger.error('Syntax error for decoded feature')
        return None
    dev,name,ctrl = p0
    decname = params[1]
    if decname not in dev.decodernames:
        logger.error('Decoder %s not yet setup for controller %s',decname,name)
        return None
    p1 = params[2].split('(')
    if len(p1) == 0:
        logger.error('Syntax error for decoded feature')
        return None
    feature= p1[0]
    if feature not in DFEATURES:
        logger.error('Unknown decoded feature (syntax error) %s',feature)
        return None
    ftargets = []
    if len(p1) == 1: 
        dargs = None
    elif '[' in p1[1]:
        lp1 = p1[1].split(']')
        for t in lp1[0].strip('[').split(','):
            if t in FTARGETS:
                ftargets.append(FTARGETS[t]())
        dargs = lp1[1].strip(')').lstrip(',')
    else:
        dargs = p1[1].strip(')')
    return dev,name,ctrl,feature,dargs,ftargets


def adddfeature(params):
    """
    Add decoded feature for controller
    params = [name:ctrl,decname,feature(ftargets,dargs)]
    """
    p = chkdfeature(params)
    if not p:
        return False
    dev,name,ctrl,feature,dargs,ftargets = p
    decname = params[1]
    lnamedf = decname + '-FEATURES'
    i = dev.dfeaturenames.index(lnamedf)
    ldfn,ldfr = dev.dfeatures[i]
    if feature in ldfn:
        logger.warning('Decoded feature has been already added %s',feature)
    ldfn.append(feature)
    args = []
    if len(ftargets) != 0 : args += [ftargets]
    if dargs: args += dargs.split(',')
    if len(args) != 0:
        ldfr.append(DFEATURES[feature](*args))
    else:
        ldfr.append(DFEATURES[feature]())
    return True


def rmdfeature(params):
    """
    Remove FEATURE from controller feature list
    """
    p = chkdfeature(params)
    if not p:
        return False
    dev,name,ctrl,feature,dargs,ftargets = p
    decname = params[1]
    lnamedf = decname + '-FEATURES'
    i = dev.dfeaturenames.index(lnamedf)
    ldfn,ldfr = dev.dfeatures[i]
    if feature not in ldfn:
        logger.error('Feature not yet added %s',feature)
        return False
    i = ldfn.index(feature)
    ldfr[i].close()		#Close coroutine generator
    ldfr.pop(i)
    ldfn.remove(feature)
    logger.info("Feature %s removed for controller %s",feature,ctrl)
    return True
